require("Helpers")

EggSpawner = EggSpawner or {}

function EggSpawner:construct()
	self.spawner = nil
	self.scoreToSpawn = 10
	self.cachedSpawnCount = 0
	self.prevScore = 0
end

function EggSpawner:OnEnable()
	self:Setup()
end

function EggSpawner:Setup()
	-- resolve
	self:ResolveSpawner()
	self:ResolveScoreToSpawn()
	
	-- set initial score state
	-- useful when switching between live and edit
	if self.scoreToSpawn == 0 then
		self:LogD("scoreToSpawn should not be 0!")
		return
	end
	self.prevScore = ScoreControl.GetCurrentScore(0)
	self.cachedSpawnCount = math.floor(self.prevScore / self.scoreToSpawn)
	
	self.attemptSpawnSound = GetGlobalSoundCue("EventModeTokenIncrement")
end

function EggSpawner:Update(dt)
	if not self.spawner then
		return
	end

	local currentScore = ScoreControl.GetCurrentScore(0)
	if currentScore < 0 then
		return
	end
	
	if self.prevScore > currentScore then
		-- reset it
		self.cachedSpawnCount = 0
	end
	
	local spawnCount = math.floor(currentScore / self.scoreToSpawn)
	if self.cachedSpawnCount < spawnCount then
		if self.attemptSpawnSound then
			self.attemptSpawnSound:Play()
		end
		local toSpawn = spawnCount - self.cachedSpawnCount
		for i = 1, toSpawn do
			self.spawner:AttemptSpawn()
		end
		
		self.cachedSpawnCount = spawnCount
	end
	
	self.prevScore = currentScore
end

function EggSpawner:ResolveSpawner()
	if self.spawner then
		return
	end
	
	self.spawner = self:GetSiblingComponentOfType("CustomSpawnerComponent")
	if not self.spawner then
		self:LogE("Could not find CustomSpawnerComponent!")
		return
	end
end

function EggSpawner:ResolveScoreToSpawn()
	self.scoreToSpawn = DataSource.GetValue("EventMode", "currentArena.Points For Token")
	self:LogD("scoreToSpawn = " .. self.scoreToSpawn)
end