require("Helpers")

EnableOnCountAndDatasourceUIComponent = EnableOnCountAndDatasourceUIComponent or {}

function EnableOnCountAndDatasourceUIComponent:construct()
	self.counterComponentPath = GetProp(self, "CounterComponent Path", "string")
	self.targetComponentPath = GetProp(self, "Target Component Path", "string")
	self.targetCount = GetProp(self, "Target Count", "int")
	self.datasourceName = GetProp(self, "DataSource Name", "string")
	self.datasourcePath = GetProp(self, "DataSource Path", "string")
	self.datasourceCountForDisable = GetProp(self, "Datasource Count For Disable", "int")
	self.onceOnly = GetProp(self, "Once Only", "bool")

	self.cachedCount = -1
end

function EnableOnCountAndDatasourceUIComponent:OnPropertyChanged(prop)
	local name = prop:GetName()
	if name == "CounterComponent Path" then
		self.counterComponentPath = GetProp(self, "CounterComponent Path", "string")
	elseif name == "Target Component Path" then
		self.targetComponentPath = GetProp(self, "Target Component Path", "string")
	elseif name == "Target Count" then
		self.targetCount = GetProp(self, "Target Count", "int")
	elseif name == "DataSource Name" then
		self.datasourceName = GetProp(self, "DataSource Name", "string")
	elseif name == "DataSource Path" then
		self.datasourcePath = GetProp(self, "DataSource Path", "string")
	elseif name == "Datasource Count For Disable" then
		self.datasourceCountForDisable = GetProp(self, "Datasource Count For Disable", "int")
	elseif name == "Once Only" then
		self.onceOnly = GetProp(self, "Once Only", "bool")
	end
end

function EnableOnCountAndDatasourceUIComponent:OnEnable()
	self:Poll()
end

function EnableOnCountAndDatasourceUIComponent:Poll()
	local counterComponent = GameCore.FindEntity(self.counterComponentPath)
	if not counterComponent then
		self:LogE("Could not find CounterComponent at " .. self.counterComponentPath)
		return
	end

	local componentId = counterComponent:GetId()
	if not componentId == "CounterComponent" then
		self:LogE("Not a CounterComponent at " .. self.counterComponentPath)
		return
	end

	if not counterComponent:IsEnabled() then
		self:LogD("CounterComponent is disabled. Ignoring.")
		return
	end

	local count = counterComponent:GetCount()

	if count < self.targetCount or (self.onceOnly and count > self.targetCount) then
		return
	end

	-- Use a datasource value to restrict the enable behaviour
	-- First used to stop preopened scrolls enabled when the user purchases 2 items
	local datasourceCount = DataSource.GetValue(self.datasourceName, self.datasourcePath)

	if not datasourceCount then
		self:LogE("DataSource value" .. self.datasourcePath .. " in datasource " .. self.datasourceName .. "could not be looked up")
	else
		local datasourceCountAsInt = datasourceCount

		if type(datasourceCount) == "string" then
			datasourceCountAsInt = tonumber(datasourceCount)
		end

		if datasourceCountAsInt >= self.datasourceCountForDisable then
			self:LogD("Not enabling component because datasource count was reached")
			return
		end
	end

	self:EnableComponent()
end

function EnableOnCountAndDatasourceUIComponent:EnableComponent()
	if IsNilOrEmpty(self.targetComponentPath) then
		self:LogW("Target Component Path is nil or empty.")
		return
	end

	local component = self:FindEntity(self.targetComponentPath)
	if not component then
		self:LogE("Could not find component at '" .. self.targetComponentPath .. "'!")
		return
	end

	SetProp(component, "enabled", "bool", true)
end