require("Helpers")

EnableOnCountUIComponent = EnableOnCountUIComponent or {}

function EnableOnCountUIComponent:construct()
	self.counterComponentPath = GetProp(self, "CounterComponent Path", "string")
	self.targetComponentPath = GetProp(self, "Target Component Path", "string")
	self.targetCount = GetProp(self, "Target Count", "int")
	self.onceOnly = GetProp(self, "Once Only", "bool")

	self.cachedCount = -1
end

function EnableOnCountUIComponent:OnPropertyChanged(prop)
	local name = prop:GetName()
	if name == "CounterComponent Path" then
		self.counterComponentPath = GetProp(self, "CounterComponent Path", "string")
	elseif name == "Target Component Path" then
		self.targetComponentPath = GetProp(self, "Target Component Path", "string")
	elseif name == "Target Count" then
		self.targetCount = GetProp(self, "Target Count", "int")
	elseif name == "Once Only" then
		self.onceOnly = GetProp(self, "Once Only", "bool")
	end
end

function EnableOnCountUIComponent:OnEnable()
	self:Poll()
end

function EnableOnCountUIComponent:Poll()
	local counterComponent = GameCore.FindEntity(self.counterComponentPath)
	if not counterComponent then
		self:LogE("Could not find CounterComponent at " .. self.counterComponentPath)
		return
	end

	local componentId = counterComponent:GetId()
	if not componentId == "CounterComponent" then
		self:LogE("Not a CounterComponent at " .. self.counterComponentPath)
		return
	end

	if not counterComponent:IsEnabled() then
		self:LogD("CounterComponent is disabled. Ignoring.")
		return
	end

	local count = counterComponent:GetCount()

	if count < self.targetCount or (self.onceOnly and count > self.targetCount) then
		return
	end

	self:EnableComponent()
end

function EnableOnCountUIComponent:EnableComponent()
	if IsNilOrEmpty(self.targetComponentPath) then
		self:LogW("Target Component Path is nil or empty.")
		return
	end

	local component = self:FindEntity(self.targetComponentPath)
	if not component then
		self:LogE("Could not find component at '" .. self.targetComponentPath .. "'!")
		return
	end

	SetProp(component, "enabled", "bool", true)
end