require("Helpers")

GhostPossessedComponent = GhostPossessedComponent or {}

function GhostPossessedComponent:construct()
	self.duration = self:CreateProperty("Duration", "float", 10.0)
	self.timer = 0
	
	self.capturePosition = Vector3(0,0,0)
	self.parent = nil
end

function GhostPossessedComponent:OnEnable()
	self:RegisterForEvent("pomegranate_thrown")
	
	local parent = self:GetParent()
	if not parent then
		self:LogE("Missing parent!")
		return
	end
	
	-- find the emitter and set up
	local emitter = parent:FindEntity("Emitter")
	if emitter then
		SetProp(emitter, "effectName", "string", "FruitSlime")
		SetProp(emitter, "enabled", "bool", true)
	end
	
	-- need to keep the fruit in place
	SetProp(self:GetParent(), "Override Fruit Position", "bool", true)
	SetProp(parent, "Override Critical", "bool", true)
	self.capturePosition = self:GetPosition()
end

function GhostPossessedComponent:Update(dt)

	if Game.IsPaused() then
		return
	end
	
	local legacyDt = Game.GetLegacyDt()
	
	self:SetPosition(self.capturePosition)
	
	self.timer = self.timer + legacyDt
	if self.timer >= self.duration:GetValue() or Game.IsGameOver() then
		self:OnExpired()
		GameCore.DestroyEntity(self)
		return
	end
end

function GhostPossessedComponent:OnExpired()
	self:UnregisterForEvent("pomegranate_thrown")
	
	local parent = self:GetParent()
	if not parent then
		return
	end
	
	-- disable the emitter
	local emitter = parent:FindEntity("Emitter")
	if emitter then
		SetProp(emitter, "enabled", "bool", false)	
	else
		self:LogE("No emitter found!")	
	end
	
	-- let go ...
	SetProp(parent, "Override Fruit Position", "bool", false)
	SetProp(parent, "Override Critical", "bool", false)
end

function GhostPossessedComponent:OnFruitSliced()
	local slicedSound = GetGlobalSoundCue("ModifierDojoGhostbustersSlice")
	if not slicedSound then
		self:LogE("Missing sound ModifierDojoGhostbustersSlice!")
	else
		slicedSound:Play()	
	end
	
	-- turn off fruit emitter
	local parent = self:GetParent()
	if parent then
		local emitter = parent:FindEntity("Emitter")
		if emitter then
			SetProp(emitter, "enabled", "bool", false)
		end
	end
end

function GhostPossessedComponent:pomegranate_thrown()
	self:OnExpired()
	GameCore.DestroyEntity(self)
end