local GAMETASKFRUITNINJA_ID = "GameTaskFruitNinja"
local AUDIO_ID = "Audio"

function GetGlobalSoundCue(name)

	-- Get a sound cue from GameTaskFruitNinja
	local gameTaskFruitNinja = GameCore.FindLiveTask(GAMETASKFRUITNINJA_ID)
	if not gameTaskFruitNinja then
		PrintString("Could not find " .. GAMETASKFRUITNINJA_ID)
		return
	end
	
	local audioRoot = gameTaskFruitNinja:FindEntity(AUDIO_ID)
	if not audioRoot then
		PrintString("Could not find " .. AUDIO_ID)
		return
	end
	
	local cue = audioRoot:FindEntity(name)
	if not cue then
		PrintString("Could not find SoundCue" .. name)
		return	
	end
	
	return cue
end

function IsNilOrEmpty(value)
	return value == nil or #value == 0
end

function GetPropertyType(typeName)
	if IsNilOrEmpty(typeName) then
		PrintString("Passed in nil or empty!")
		return nil
	end
	
	-- As defined in GameCoreEntity_Binding.cpp
	if typeName == "bool" then
		return 1
	elseif typeName == "int" then
		return 7
	elseif typeName == "uint" then
		return 10
	elseif typeName == "float" then
		return 5
	elseif typeName == "string" then
		return 2
	elseif typeName == "rotation" then
		return 6
	elseif typeName == "colour" then
		return 8
	elseif typeName == "vector2" then
		return 4
	elseif typeName == "vector3" then
		return 3
	elseif typeName == "vector4" then
		return 9
	else
		return nil
	end
end

function GetProp(target, propertyName, typeName)
	if not target then
		PrintString("Target (self) is nil!")
		return nil
	end

	if IsNilOrEmpty(propertyName) then
		PrintString("propertyName is nil or empty!")
		return nil
	end	
	
	if IsNilOrEmpty(typeName) then
		PrintString("typeName is nil!")
		return nil
	end	
	
	local t = GetPropertyType(typeName)
	if not t then
		PrintString(typeName .. " is not a valid type!")
		return nil
	end
	
	local returnedType, value = target:GetPropertyEx(propertyName, t)
	return value
end

function SetProp(target, propertyName, typeName, value)
	if not target then
		PrintString("Target (self) is nil!")
		return nil
	end
	
	if IsNilOrEmpty(propertyName) then
		PrintString("propertyName is nil or empty!")
		return nil
	end	
	
	if IsNilOrEmpty(typeName) then
		PrintString("typeName is nil!")
		return nil
	end	
	
	local t = GetPropertyType(typeName)
	if not t then
		PrintString(typeName .. " is not a valid type!")
		return nil
	end

	target:SetPropertyEx(t, propertyName, value)
end

function Lerp(a, b, t)
	return a + (b - a) * t
end