require("Helpers")

NPSRatingComponent = NPSRatingComponent or {}

local SLIDERBAR_VALUE_PROPERTY = "sliderSnapIndexOne"
local SAVEFILE = "SaveFile"
local RATINGS_OBJECT = "userRatings"
local RATING = "rating"
local CHOICE = "choice"
local POPUPS_SEEN = "PopupsSeen"

function NPSRatingComponent:construct()
	self.slideBarPath = self:CreateProperty("Slide Bar Path", "string", "")
	self.npsId = self:CreateProperty("NPS Id", "string", "")
end

function NPSRatingComponent:OnSubmitPressed()
	self:LogD("OnSubmitPressed")
	
	local slideBar = UserInterfaceManager:FindComponent(self.slideBarPath:GetValue())
	if not slideBar then
		return
	end
	
	local value = GetProp(slideBar, SLIDERBAR_VALUE_PROPERTY, "uint")
	if not value then
		self:LogE("Value is nil for " .. SLIDERBAR_VALUE_PROPERTY)
		return
	end
	
	self:Save("Submitted", value + 1)
end

function NPSRatingComponent:OnMaybeLaterPressed()
	self:LogD("OnMaybeLayerPressed")
	self:Save("MaybeLater")
end

function NPSRatingComponent:OnNoThanksPressed()
	self:LogD("OnNoThanksPressed")
	self:Save("NoThanks")
end

function NPSRatingComponent:Save(choice, rating)
	if IsNilOrEmpty(choice) then
		self:LogE("Choice is nil or empty!")
		return
	end
	
	if IsNilOrEmpty(self.npsId:GetValue()) then
		self:LogE("NPS Id is nil or empty!")
		return
	end
	
	if IsNilOrEmpty(RATINGS_OBJECT) then
		self:LogE("RATINGS_OBJECT is nil or empty!")
		return
	end
	
	-- get user first
	local user = ServiceManager:GetDefaultUser()
	if not user then
		self:LogE("User is nil!")
		return
	end

	local cloudObject = CloudService.GetUserObject(user, SAVEFILE)
	if not cloudObject then
		self:LogE(SAVEFILE .. " is nil!")
		return
	end

	local npsObj = cloudObject:GetValueAtPath(RATINGS_OBJECT .. "." .. self.npsId:GetValue())
	local choiceObj = npsObj:GetMember(CHOICE)
	cloudObject:ConditionalSetNewer(choiceObj, choice)
	
	if rating ~= nil then
		local ratingObj = npsObj:GetMember(RATING)
		cloudObject:ConditionalSetNewer(ratingObj, rating)
	end
	
	cloudObject:SaveLocalChanges()
end

function NPSRatingComponent:Reset()
	self:LogD("Resetting...")
	
	-- Reset the 'seen' data
	-- get user first
	local user = ServiceManager:GetDefaultUser()
	if not user then
		self:LogE("User is nil!")
		return
	end
	
	local cloudObject = CloudService.GetUserObject(user, SAVEFILE)
	if not cloudObject then
		self:LogE(SAVEFILE .. " is nil!")
		return
	end
	
	if not IsNilOrEmpty(self.npsId:GetValue()) then
		local hasSeen = cloudObject:GetValueAtPath(POPUPS_SEEN .. "." .. self.npsId:GetValue())
		cloudObject:ConditionalSetNewer(hasSeen, false)
		
		if not IsNilOrEmpty(RATINGS_OBJECT) then
			local npsObj = cloudObject:GetValueAtPath(self.npsId:GetValue())
			cloudObject:ConditionalSetNewer(npsObj, nil)
		end

		cloudObject:SaveLocalChanges()
	else
		self:LogW("NPS Id is nil or empty!")
	end
end