require("Helpers")

OnEnableSaveCloudValueComponent = OnEnableSaveCloudValueComponent or {}

function OnEnableSaveCloudValueComponent:construct()
	self.cloudObjectId = self:CreateProperty("Cloud Object Id", "string", "")
	self.path = self:CreateProperty("Path", "string", "")
	self.value = self:CreateProperty("Value", "string", "")
	self.type = self:CreateProperty("Type", "string", "")
end

function OnEnableSaveCloudValueComponent:OnEnable()
	if IsNilOrEmpty(self.cloudObjectId:GetValue()) then
		self:LogW("Attempted to save with empty 'Cloud Object Id'.")
		return
	end

	if IsNilOrEmpty(self.path:GetValue()) then
		self:LogW("Attempted to save with empty 'Path'.")
		return
	end

	if IsNilOrEmpty(self.type:GetValue()) then
		self:LogW("Attempted to save with empty 'Type'.")
		return
	end

	local user = ServiceManager.GetDefaultUser()
	if not user then
		self:LogE("User is nil!")
		return false
	end

	local cloudObject = CloudService.GetUserObject(user, self.cloudObjectId:GetValue())
	if not cloudObject then
		self:LogE("Cloud object '" .. elf.cloudObjectId:GetValue() .. "' is nil!")
		return false
	end

	local smartValue = cloudObject:GetValueAtPath(self.path:GetValue())
	if self.type:GetValue() == "string" then
		cloudObject:ConditionalSetNewer(smartValue, self.value:GetValue())
	elseif self.type:GetValue() == "int" then
		cloudObject:ConditionalSetNewer(smartValue, tonumber(self.value:GetValue()))
	elseif self.type:GetValue() == "bool" then
		cloudObject:ConditionalSetNewer(smartValue, self.value:GetValue() == "true" or self.value:GetValue() == "True")
	else
		self:LogE("Attempted to save with unknown type '" .. self.type:GetValue() .. "'.'")
		return
	end

	cloudObject:SaveLocalChanges()
end