require("Helpers")

PopupLauncherComponent = PopupLauncherComponent or {}

local SAVEFILE = "SaveFile"
local SAVEFILE_MEMBER = "PopupsSeen"

function PopupLauncherComponent:construct()
	self.screenName = self:CreateProperty("Screen Name", "string", "")
	self.popupSeenId = self:CreateProperty("Popup Seen Id", "string", "")
	self.onceOnly = self:CreateProperty("Once Only", "bool", true)
	self.counter = self:CreateProperty("Counter", "int", 0)
	self.counterTarget = self:CreateProperty("Counter Target", "int", 3)
end

function PopupLauncherComponent:OnPropertyChanged(prop)
	if prop == self.counter then
		self:AttemptShow()
	end
end

function PopupLauncherComponent:AttemptShow()
	if IsNilOrEmpty(self.screenName:GetValue()) then
		self:LogE("Screen Name is nil or empty!")
		self:Disable()
		return
	end
	
	if self.counter:GetValue() < self.counterTarget:GetValue() then
		self:LogD("Target not met yet.")
		return
	end

	if not self:IsEnabled() then
		return
	end

	if self.onceOnly:GetValue() and self:HasSeen() then
		return
	end
	
	UserInterfaceManager:DisplayScreen(self.screenName:GetValue())
	self:SaveSeen()
	self:Disable()
end

function PopupLauncherComponent:HasSeen()
	if IsNilOrEmpty(self.popupSeenId:GetValue()) then
		return false
	end
	
	if not Game.IsInitialised() then
		self:LogW("Game is not initialised yet. Cannot do Bricknet logic.")
		return
	end
	
	-- cloud value check
	-- get user first
	local user = ServiceManager:GetDefaultUser()
	if not user then
		self:LogE("User is nil!")
		return false
	end
	
	local cloudObject = CloudService.GetUserObject(user, SAVEFILE)
	if not cloudObject then
		self:LogE(SAVEFILE .. " is nil!")
		return false
	end
	
	local value = cloudObject:GetValue()
	if not value then
		self:LogE(SAVEFILE .. " value is nil!")
		return false
	end
	
	local popupsSeen = value:GetMember(SAVEFILE_MEMBER)
	if not popupsSeen then
		self:LogE(SAVEFILE_MEMBER .. " value is nil!")
		return false
	end
	
	local hasSeen = popupsSeen:GetMember(self.popupSeenId:GetValue())
	return hasSeen:AsBool()
end

function PopupLauncherComponent:SaveSeen()
	if IsNilOrEmpty(self.popupSeenId) then
		return
	end
	
	if not Game.IsInitialised() then
		self:LogW("Game is not initialised yet. Cannot do Bricknet logic.")
		return
	end
	
	-- cloud value check
	-- get user first
	local user = ServiceManager:GetDefaultUser()
	if not user then
		self:LogE("User is nil!")
		return
	end
	
	local cloudObject = CloudService.GetUserObject(user, SAVEFILE)
	if not cloudObject then
		self:LogE(SAVEFILE .. " is nil!")
		return
	end
	
	local value = cloudObject:GetValue()
	if not value then
		self:LogE(SAVEFILE .. " value is nil!")
		return
	end
	
	local popupsSeen = value:GetMember(SAVEFILE_MEMBER)
	local hasSeen = popupsSeen:GetMember(self.popupSeenId:GetValue())
	cloudObject:ConditionalSetNewer(hasSeen, true)
	cloudObject:SaveLocalChanges()
end