require("Helpers")

RevertablePropertyComponent = RevertablePropertyComponent or {}

function RevertablePropertyComponent:construct()
	self.targetComponent = self:CreateProperty("Target Component", "string", "")
	self.targetProperty = self:CreateProperty("Target Property", "string", "")
	self.propertyType = self:CreateProperty("Property Type", "string", "")
	self.propertyValue = self:CreateProperty("Property Value", "string", "")
	self.applied = self:CreateProperty("Applied", "bool", false)
end

function RevertablePropertyComponent:OnEnable()
	self:Apply()
end

function RevertablePropertyComponent:OnDisable()
	self:Revert()
end

function RevertablePropertyComponent:Apply()
	if self.applied:GetValue() then
		return
	end
	
	if IsNilOrEmpty(self.targetComponent:GetValue()) then
		return
	end
	
	if IsNilOrEmpty(self.targetProperty:GetValue()) or 
	   IsNilOrEmpty(self.propertyType:GetValue()) or
	   IsNilOrEmpty(self.propertyValue:GetValue()) then
		self:LogE("Target Property, Property Type or Property Value is empty!")
		return		
	end
	
	local component = GameCore.FindEntity(self.targetComponent:GetValue())
	if not component then
		self:LogE("Could not find component at " .. self.targetComponent:GetValue() .. "!")
		return
	end
	
	local prevValue = GetProp(component, self.targetProperty:GetValue(), self.propertyType:GetValue())
	if not prevValue then
		self:LogE("Could not find target property for " .. self.targetComponent:GetValue() .. " with name " .. self.targetProperty:GetValue() .. " and type " .. self.propertyType:GetValue() .. "!")
		return	
	end
	
	-- quickly handling properties that are needed right now, can extend later
	if self.propertyType:GetValue() == "float" or self.propertyType:GetValue() == "int" or self.propertyType:GetValue() == "uint" then
		SetProp(component, self.targetProperty:GetValue(), self.propertyType:GetValue(), tonumber(self.propertyValue:GetValue()))
	elseif self.propertyType:GetValue() == "string" then
		SetProp(component, self.targetProperty:GetValue(), self.propertyType:GetValue(), self.propertyValue:GetValue())
	else
		self:LogE("Apply: Unhandled property " .. self.targetProperty:GetValue() .. " of type " .. self.propertyType:GetValue() .. " for .. " .. self.targetComponent:GetValue() .. "!")
		return
	end
	
	-- store for later
	self.prevValue = prevValue
	self.applied:SetValue(true)
end

function RevertablePropertyComponent:Revert()
	if not self.applied:GetValue() then
		return
	end
	
	if not self.prevValue then
		self:LogE("Previous value was not recorded!")
		return
	end
	
	if IsNilOrEmpty(self.targetComponent:GetValue()) then
		return
	end
	
	if IsNilOrEmpty(self.targetProperty:GetValue()) or 
	   IsNilOrEmpty(self.propertyType:GetValue()) then
		self:LogE("Target Property, Property Type or Property Value is empty!")
		return		
	end

	local component = GameCore.FindEntity(self.targetComponent:GetValue())
	if not component then
		self:LogE("Could not find component at " .. self.targetComponent:GetValue() .. "!")
		return
	end
	
	if self.propertyType:GetValue() == "float" or self.propertyType:GetValue() == "int" or self.propertyType:GetValue() == "uint" then
		SetProp(component, self.targetProperty:GetValue(), self.propertyType:GetValue(), self.prevValue)
	elseif self.propertyType:GetValue() == "string" then
		SetProp(component, self.targetProperty:GetValue(), self.propertyType:GetValue(), self.prevValue)
	else
		self:LogE("Revert: Unhandled property " .. self.targetProperty:GetValue() .. " of type " .. self.propertyType:GetValue() .. " for .. " .. self.targetComponent:GetValue() .. "!")
	end
	
	self.prevValue = nil
	self.applied:SetValue(false)
end
