require("Helpers")

RewardedAdUIComponent = RewardedAdUIComponent or {}

function RewardedAdUIComponent:construct()
	self.adName = GetProp(self, "Ad Name", "string")
	self.onPreparedTrigger = GetProp(self, "On Prepared Trigger", "string")
	self.prepareSuccessTrigger = GetProp(self, "Prepare Success Trigger", "string")
	self.prepareFailTrigger = GetProp(self, "Prepare Fail Trigger", "string")
	self.upsellSuccessTrigger = GetProp(self, "Upsell Success Trigger", "string")
	self.upsellFailTrigger = GetProp(self, "Upsell Fail Trigger", "string")
	self.watchSuccessTrigger = GetProp(self, "Watch Success Trigger", "string")
	self.watchFailTrigger = GetProp(self, "Watch Fail Trigger", "string")
end

function RewardedAdUIComponent:OnPropertyChanged(prop)
	local name = prop:GetName()
	if name == "Ad Name" then
		self.adName = GetProp(self, "Ad Name", "string")
	elseif name == "On Prepared Trigger" then
		self.onPreparedTrigger = GetProp(self, "On Prepared Trigger", "string")
	elseif name == "Prepare Success Trigger" then
		self.prepareSuccessTrigger = GetProp(self, "Prepare Success Trigger", "string")
	elseif name == "Prepare Success Trigger" then
		self.prepareFailTrigger = GetProp(self, "Prepare Fail Trigger", "string")
	elseif name == "Upsell Success Trigger" then
		self.upsellSuccessTrigger = GetProp(self, "Upsell Success Trigger", "string")
	elseif name == "Upsell Success Trigger" then
		self.upsellFailTrigger = GetProp(self, "Upsell Fail Trigger", "string")
	elseif name == "Watch Success Trigger" then
		self.watchSuccessTrigger = GetProp(self, "Watch Success Trigger", "string")
	elseif name == "Watch Success Trigger" then
		self.watchFailTrigger = GetProp(self, "Watch Fail Trigger", "string")
	end
end

function RewardedAdUIComponent:GetAdComponent()
	if IsNilOrEmpty(self.adName) then
		self:LogW("Ad Name is empty.")
		return nil
	end
	
	local adPath = self.adName .. ".RewardedAdComponent"
	
	local task = GameCore.FindLiveTask("GameTaskFruitNinja")
	if not task then
		self:LogE("Unable to find GameTaskFruitNinja!")
		return nil
	end
	
	local rewardedAds = task:FindEntity("@RewardedAdsComponent")
	if not rewardedAds then
		self:LogE("Unable to find RewardedAdsComponent in GameTaskFruitNinja!")
		return nil
	end
	
	local actor = rewardedAds:GetParentActor()
	if not actor then
		self:LogE("Unable to get actor of RewardedAdsComponent in GameTaskFruitNinja!")
		return nil		
	end
	
	local rewardedAd = actor:FindEntity(adPath)
	if not rewardedAd then
		self:LogE("Unable to find RewardedAdComponent at " .. adPath)
		return nil	
	end
	
	return rewardedAd
end

function RewardedAdUIComponent:IsPrepared()
	local rewardedAd = self:GetAdComponent()
	
	if not rewardedAd then
		return false
	end
	
	return rewardedAd:IsPrepared()
end

function RewardedAdUIComponent:PrepareAd()
	local rewardedAd = self:GetAdComponent()
	
	if not rewardedAd then
		return
	end
	
	rewardedAd:PrepareAd(self, self.prepareSuccessTrigger, self.prepareFailTrigger)
end

function RewardedAdUIComponent:UpsellAd()
	local rewardedAd = self:GetAdComponent()
	
	if not rewardedAd then
		return
	end
	
	rewardedAd:UpsellAd(self, self.upsellSuccessTrigger, self.upsellFailTrigger)
end

function RewardedAdUIComponent:ShowAd()
	local rewardedAd = self:GetAdComponent()
	
	if not rewardedAd then
		return
	end
	
	rewardedAd:ShowAd(self, self.watchSuccessTrigger, self.watchFailTrigger)
end