require("Helpers")

SaveTutorialStepDurationComponent = SaveTutorialStepDurationComponent or {}

local CLOUDOBJECT_ID = "Tutorial"
local OBJECTIVES_KEY = "objs"
local TIME_KEY = "time"
local TUTORIALCOMPONENT_ID = "TutorialComponent"
local TUTORIALOBJECTIVE_ID = "TutorialObjective"
local STOPWATCHCOMPONENT_ID = "StopWatchComponent"

function SaveTutorialStepDurationComponent:construct()
	self.useAppStart = self:CreateProperty("Use App Start", "bool", false)
end

function SaveTutorialStepDurationComponent:OnDisable()
	-- get tutorial name
	local tutorialName = self:GetTutorialName()
	if IsNilOrEmpty(tutorialName) then
		self:LogE("Tutorial name is nil or empty!")
		return
	end

	-- get tutorial objective name
	local objectiveName = self:GetTutorialObjectiveName()
	if IsNilOrEmpty(objectiveName) then
		self:LogE("Tutorial objective name is nil or empty!")
		return
	end

	local recordedTime = 0
	if self.useAppStart:GetValue() then
		-- get time since app start
		recordedTime = self:GetTimeSinceAppStart()
		if not recordedTime then
			self:LogE("Time since app start is nil!")
			return
		end
	else
		-- get stopwatch last recorded time 
		recordedTime = self:GetLastRecordedTime()
		if not recordedTime then
			self:LogE("Last recorded time is nil!")
			return
		end
	end

	-- save to cloud
	local user = ServiceManager.GetDefaultUser()
	if not user then
		self:LogE("User is nil!")
		return
	end

	local cloudObject = CloudService.GetUserObject(user, CLOUDOBJECT_ID)
	if not cloudObject then
		self:LogE("Cloud object '" .. CLOUDOBJECT_ID .. "' is nil!")
		return false
	end

	local path = tutorialName .. "." .. OBJECTIVES_KEY .. "." .. objectiveName .. "." .. TIME_KEY
	self:LogD("Saving time '" .. recordedTime .. " at " .. CLOUDOBJECT_ID .. "." .. path)
	local smartValue = cloudObject:GetValueAtPath(path)
	cloudObject:ConditionalSetNewer(smartValue, recordedTime)
	cloudObject:SaveLocalChanges()
end

function SaveTutorialStepDurationComponent:GetTutorialName()
	-- find first ancestor with TutorialComponent
	local actor = self:GetParentActor()
	while actor do
		-- check if it has TutorialComponent
		local tutorialComponent = actor:FindEntity(TUTORIALCOMPONENT_ID)
		if tutorialComponent then
			-- return this actor id
			return actor:GetId()
		end

		-- go up the tree
		actor = actor:GetParent()
	end

	return nil
end

function SaveTutorialStepDurationComponent:GetTutorialObjectiveName()
	-- find first ancestor with TutorialObjective
	local actor = self:GetParentActor()
	while actor do
		-- check if it has TutorialObjective
		local tutorialObjectiveComponent = actor:FindEntity(TUTORIALOBJECTIVE_ID)
		if tutorialObjectiveComponent then
			-- return this actor id
			return actor:GetId()
		end

		-- go up the tree
		actor = actor:GetParent()
	end

	return nil
end

function SaveTutorialStepDurationComponent:GetLastRecordedTime()
	-- should be siblinh component
	local stopWatchComponent = self:GetSiblingComponentOfType(STOPWATCHCOMPONENT_ID)
	if not stopWatchComponent then
		self:LogE("Could not find '" .. STOPWATCHCOMPONENT_ID .. "'!")
		return nil
	end

	return stopWatchComponent:GetLastRecordedTime()
end

function SaveTutorialStepDurationComponent:GetTimeSinceAppStart()
	-- should be siblinh component
	local stopWatchComponent = self:GetSiblingComponentOfType(STOPWATCHCOMPONENT_ID)
	if not stopWatchComponent then
		self:LogE("Could not find '" .. STOPWATCHCOMPONENT_ID .. "'!")
		return nil
	end

	return stopWatchComponent:GetTimeSinceAppStart()
end