require("Helpers")

OrderedSelectorComponent = OrderedSelectorComponent or {}

local SELECTABLECOMPONENT_ID = "SelectableComponent"

function OrderedSelectorComponent:construct()
	self.pendingSelect = false
	self.currentlySelected = nil
end

function OrderedSelectorComponent:OnPropertyChanged(prop)
	
end

function OrderedSelectorComponent:OnEnable()
	self.pendingSelect = true
end

function OrderedSelectorComponent:Update(dt)
	if self.pendingSelect then
		self:Select()
	end
end

function OrderedSelectorComponent:Select()
	self.pendingSelect = false

	-- Get parent actor
	local actor = self:GetParentActor()
	if not actor then
		self:LogE("Parent actor is nil!")
		self:Disable()
		return
	end

	-- Get all children Actors with SelectableComponent
	local children = actor:GetChildren()
	if not children then
		self:Disable()
		return
	end

	local childCount = #children
	if childCount == 0 then
		self:Disable()
		return
	end

	local selectableComponentList = {}

	-- Disable them all
	for i = 1, childCount do
		local child = children[i]

		-- check if Actor	
		local childType = child:GetType()

		if childType == "Actor" or childType == "ActorExtended" then
			local selectableComponent = child:FindEntity(SELECTABLECOMPONENT_ID)
			if selectableComponent then
				table.insert(selectableComponentList, selectableComponent)

				-- disable
				selectableComponent:SetParentActorEnabled(false)
			end
		end	
	end

	local selectableComponentsCount = #selectableComponentList

	if selectableComponentsCount == 0 then
		self:Disable()
		return
	end

	if not self.currentlySelected or self.currentlySelected:IsReleased() then
		-- choose the first one
		self.currentlySelected = selectableComponentList[1]
	else

		for i = 1, selectableComponentsCount do
			local selectableComponent = selectableComponentList[i]

			if selectableComponent:IsSameEntity(self.currentlySelected) then
				-- choose the next one
				if i == selectableComponentsCount then
					-- wrap
					self.currentlySelected = selectableComponentList[1]
				else
					self.currentlySelected = selectableComponentList[i + 1]
				end

				break
			end
		end
	end

	if not self.currentlySelected then
		self:LogE("None currently selected!")
	else
		self.currentlySelected:SetParentActorEnabled(true)
	end

	-- once done, disable
	self:Disable()
end