require("Helpers")

RandomSelectorComponent = RandomSelectorComponent or {}

local SELECTABLECOMPONENT_ID = "SelectableComponent"

function RandomSelectorComponent:construct()
	self.pendingSelect = false
end

function RandomSelectorComponent:OnPropertyChanged(prop)
	
end

function RandomSelectorComponent:OnEnable()
	self.pendingSelect = true
end

function RandomSelectorComponent:Update(dt)
	if self.pendingSelect then
		self:Select()
	end
end

function RandomSelectorComponent:Select()
	self.pendingSelect = false

	-- Get parent actor
	local actor = self:GetParentActor()
	if not actor then
		self:LogE("Parent actor is nil!")
		self:Disable()
		return
	end

	-- Get all children Actors with SelectableComponent
	local children = actor:GetChildren()
	if not children then
		self:Disable()
		return
	end

	local childCount = #children
	if childCount == 0 then
		self:Disable()
		return
	end

	local selectableComponentList = {}
	local totalWeight = 0

	-- Disable them all
	for i = 1, childCount do
		local child = children[i]

		-- check if Actor	
		local childType = child:GetType()

		if childType == "Actor" or childType == "ActorExtended" then
			local selectableComponent = child:FindEntity(SELECTABLECOMPONENT_ID)
			if selectableComponent then
				totalWeight = totalWeight + selectableComponent.weight
				table.insert(selectableComponentList, selectableComponent)

				-- disable
				selectableComponent:SetParentActorEnabled(false)
			end
		end	
	end

	local selectableComponentsCount = #selectableComponentList

	if selectableComponentsCount == 0 then
		self:Disable()
		return
	end

	if totalWeight == 0 then
		self:LogW("Calculated total weight is 0.")
		return
	end

	local currentWeight = 0
	local randomWeight = math.random(totalWeight)
	for i = 1, selectableComponentsCount do
		local selectableComponent = selectableComponentList[i]

		currentWeight = currentWeight + selectableComponent.weight
		if currentWeight >= randomWeight then
			selectableComponent:SetParentActorEnabled(true)
			break
		end
	end

	-- once done, disable
	self:Disable()
end