require("Helpers")

SetPropertyFromCloudComponent = SetPropertyFromCloudComponent or {}

function SetPropertyFromCloudComponent:construct()
	self.cloudObjectName = self:CreateProperty("Cloud Object Name", "string", "")
	self.path = self:CreateProperty("Path", "string", "")
	self.cloudValue = nil
	self.propertyType = self:CreateProperty("Property Type", "string", "int")
	
	self.targetComponentPath = self:CreateProperty("Target Component", "string", "")
	self.targetComponent = nil
	self.targetProperty = self:CreateProperty("Target Property", "string", "")
end

function SetPropertyFromCloudComponent:OnPropertyChanged(prop)
	if prop == self.targetProperty then
		self:ResolveTargetComponent()
	end
end

function SetPropertyFromCloudComponent:OnEnable()
	self:ResolveTargetComponent()
end

function SetPropertyFromCloudComponent:Update()
	if not self.targetComponent then
		return
	end
	
	self:ResolveCloudValue()
	
	if not self.cloudValue then
		return
	end
	
	SetProp(self.targetComponent, self.targetProperty:GetValue(), self.propertyType:GetValue(), self.cloudValue)
end

function SetPropertyFromCloudComponent:ResolveTargetComponent()
	if IsNilOrEmpty(self.targetComponentPath:GetValue()) then
		self.targetComponent = nil
		return
	end
	
	self.targetComponent = self:FindEntity(self.targetComponentPath:GetValue())
	if not self.targetComponent then
		self:LogE("Unable to find target component at " .. self.targetComponentPath:GetValue())
	end
end

function SetPropertyFromCloudComponent:ResolveCloudValue()
	if not Game.IsInitialised() then
		return
	end
	
	if IsNilOrEmpty(self.cloudObjectName:GetValue()) or IsNilOrEmpty(self.path:GetValue()) then
		return
	end
	
	if IsNilOrEmpty(self.propertyType:GetValue()) then
		self:LogE("Property Type not specified!")
		return
	end
	
	local user = ServiceManager:GetDefaultUser()
	if not user then
		self:LogE("User is nil!")
		return
	end
	
	local cloudObject = CloudService.GetUserObject(user, self.cloudObjectName:GetValue())
	if not cloudObject then
		self:LogE(self.cloudObjectName:GetValue() .. " is nil!")
		return false
	end
	
	local smartValue = cloudObject:GetValueAtPath(self.path:GetValue())
	if not smartValue then
		self:LogE(self.cloudObjectName:GetValue() .. " value is nil!")
		return false
	end
	
	if self.propertyType:GetValue() == "int" then
		self.cloudValue = smartValue:AsInt()
	end
end