require("Helpers")

SetPropertyFromDatasourceComponent = SetPropertyFromDatasourceComponent or {}

function SetPropertyFromDatasourceComponent:construct()
	self.dataSourceName = self:CreateProperty("Datasource Name", "string", "")
	self.dataPath = self:CreateProperty("Datasource Path", "string", "")
	self.propertyType = self:CreateProperty("Property Type", "string", "")
	self.useLocalTargetComponentPath = self:CreateProperty("Use Local Component Path", "bool", true)
	self.targetComponentPath = self:CreateProperty("Target Component", "string", "")
	self.targetProperty = self:CreateProperty("Target Property", "string", "")

	self.targetComponent = nil
	self.datasourceValue = nil
end

function SetPropertyFromDatasourceComponent:OnPropertyChanged(prop)
	if prop == self.dataSourceName or
	   prop == self.dataPath or
	   prop == self.useLocalTargetComponentPath or
	   prop == self.targetComponentPath or 
	   prop == self.targetProperty then
		self:ResolveTargetComponent()
	end

	if not self:IsEnabled() then
		return
	end
	
	self:ApplySetProperty()
end

function SetPropertyFromDatasourceComponent:OnEnable()
	self:ResolveTargetComponent()
	self:ResolveDatasourceValue()

	self:ApplySetProperty()
	
	self:RegisterForEvent("OnDataModified")
end

function SetPropertyFromDatasourceComponent:OnDisable()
	self:UnregisterForEvent("OnDataModified")
end

function SetPropertyFromDatasourceComponent:ApplySetProperty()
	if not self.targetComponent then
		self:LogE("Target Component is nil")
		return
	end

	if not self.datasourceValue then
		self:LogE("Datasource value is nil")
		return
	end

	SetProp(self.targetComponent, self.targetProperty:GetValue(), self.propertyType:GetValue(), self.datasourceValue)
	self:LogD("Property change applied")
end

function SetPropertyFromDatasourceComponent:ResolveTargetComponent()
	if IsNilOrEmpty(self.targetComponentPath:GetValue()) then
		self.targetComponent = nil
		return
	end
	
	if self.useLocalTargetComponentPath:GetValue() then
		self.targetComponent = self:FindEntity(self.targetComponentPath:GetValue())
	else
		self.targetComponent = GameCore.FindEntity(self.targetComponentPath:GetValue())
	end
	
	if not self.targetComponent then
		self:LogE("Unable to find target component at " .. self.targetComponentPath:GetValue())
	end
end

function SetPropertyFromDatasourceComponent:ResolveDatasourceValue()
	self.datasourceValue = DataSource.GetValue(self.dataSourceName:GetValue(), self.dataPath:GetValue())
end

function SetPropertyFromDatasourceComponent:OnDataModified(ev)
	if ev:GetString("name") ~= self.dataSourceName:GetValue() then
		return
	end
	
	self:ResolveDatasourceValue()
	self:ApplySetProperty()
end