require("Helpers")

TriggerOnEnabledFlagComponent = TriggerOnEnabledFlagComponent or {}

function TriggerOnEnabledFlagComponent:construct()
	self.onEnable = self:CreateProperty("On Enable", "bool", true)
	self.onDisable = self:CreateProperty("On Disable", "bool", false)
	self.offThenOn = self:CreateProperty("Off Then On", "bool", false)
	self.withComponent = self:CreateProperty("With Component", "string", "") 
end

function TriggerOnEnabledFlagComponent:OnEnable()
	if not self.onEnable:GetValue() then
		return
	end

	self:EnableChildren()
end

function TriggerOnEnabledFlagComponent:OnDisable()
	if not self.onDisable:GetValue() then
		return
	end

	self:EnableChildren()
end

function TriggerOnEnabledFlagComponent:EnableChildren()
	-- get actor children
	local actor = self:GetParentActor()
	if not actor then
		self:LogE("Could not find actor!")
		return
	end
	
	local children = actor:GetChildren()
	if not children then
		self:LogE("Unable to get children!")
		return
	end
	
	-- disable all actors
	local count = #children
	for i = 1, count do
		local child = children[i]
		
		local childType = child:GetType()
		
		if childType == "Actor" or childType == "ActorExtended" then
			
			if IsNilOrEmpty(self.withComponent:GetValue()) then
				if self.offThenOn:GetValue() then
					SetProp(child, "enabled", "bool", false)
				end
				SetProp(child, "enabled", "bool", true)
			else
				local component = child:FindEntity(self.withComponent:GetValue())
				if component then
					if self.offThenOn:GetValue() then
						component:SetParentActorEnabled(false)
					end
					component:SetParentActorEnabled(true)
				end			
			end
		end
	end
end